# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: minknow/rpc/acquisition.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='minknow/rpc/acquisition.proto',
  package='ont.rpc.acquisition',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x1dminknow/rpc/acquisition.proto\x12\x13ont.rpc.acquisition\x1a\x1fgoogle/protobuf/timestamp.proto\"\x99\x03\n\x0cStartRequest\x12\x1d\n\x15wait_until_processing\x18\x01 \x01(\x08\x12\"\n\x1a\x64ont_wait_for_device_ready\x18\x02 \x01(\x08\x12\x34\n\x0fgenerate_report\x18\x03 \x01(\x0e\x32\x1b.ont.rpc.acquisition.Option\x12\x41\n\x1csend_sequencing_read_metrics\x18\x04 \x01(\x0e\x32\x1b.ont.rpc.acquisition.Option\x12=\n\x18send_basecalling_metrics\x18\x05 \x01(\x0e\x32\x1b.ont.rpc.acquisition.Option\x12-\n\x07purpose\x18\x06 \x01(\x0e\x32\x1c.ont.rpc.acquisition.Purpose\x12-\n\x08\x61nalysis\x18\x07 \x01(\x0e\x32\x1b.ont.rpc.acquisition.Option\x12\x30\n\x0b\x66ile_output\x18\x08 \x01(\x0e\x32\x1b.ont.rpc.acquisition.Option\"\x1f\n\rStartResponse\x12\x0e\n\x06run_id\x18\x01 \x01(\t\"\xdc\x01\n\x0bStopRequest\x12H\n\x13\x64\x61ta_action_on_stop\x18\x01 \x01(\x0e\x32+.ont.rpc.acquisition.StopRequest.DataAction\x12\x18\n\x10wait_until_ready\x18\x02 \x01(\x08\x12\x15\n\rkeep_power_on\x18\x03 \x01(\x08\"R\n\nDataAction\x12\x10\n\x0cSTOP_DEFAULT\x10\x00\x12\x16\n\x12STOP_KEEP_ALL_DATA\x10\x01\x12\x1a\n\x16STOP_FINISH_PROCESSING\x10\x02\"\x0e\n\x0cStopResponse\"+\n\x1bWatchForStatusChangeRequest\x12\x0c\n\x04stop\x18\x01 \x01(\x08\"R\n\x1cWatchForStatusChangeResponse\x12\x32\n\x06status\x18\x01 \x01(\x0e\x32\".ont.rpc.acquisition.MinknowStatus\"\x16\n\x14\x43urrentStatusRequest\"K\n\x15\x43urrentStatusResponse\x12\x32\n\x06status\x18\x01 \x01(\x0e\x32\".ont.rpc.acquisition.MinknowStatus\"\x14\n\x12GetProgressRequest\"\x9c\x01\n\x13GetProgressResponse\x12O\n\x0fraw_per_channel\x18\x01 \x01(\x0b\x32\x36.ont.rpc.acquisition.GetProgressResponse.RawPerChannel\x1a\x34\n\rRawPerChannel\x12\x10\n\x08\x61\x63quired\x18\x01 \x01(\x04\x12\x11\n\tprocessed\x18\x02 \x01(\x04\".\n\x1cGetAcquisitionRunInfoRequest\x12\x0e\n\x06run_id\x18\x01 \x01(\t\"\xa9\x03\n\x17\x41\x63quisitionYieldSummary\x12\x12\n\nread_count\x18\x01 \x01(\x03\x12\x1a\n\x12written_read_count\x18\x02 \x01(\x03\x12\"\n\x1a\x62\x61secalled_pass_read_count\x18\x03 \x01(\x03\x12\"\n\x1a\x62\x61secalled_fail_read_count\x18\x04 \x01(\x03\x12%\n\x1d\x62\x61secalled_skipped_read_count\x18\n \x01(\x03\x12\x18\n\x10\x62\x61secalled_bases\x18\x05 \x01(\x03\x12\x1a\n\x12\x62\x61secalled_samples\x18\x06 \x01(\x03\x12\x1c\n\x14selected_raw_samples\x18\x07 \x01(\x03\x12\x17\n\x0fselected_events\x18\x08 \x01(\x03\x12 \n\x18\x65stimated_selected_bases\x18\t \x01(\x03\x12\x1f\n\x17\x62ytes_to_write_produced\x18\x0b \x01(\x03\x12\x1d\n\x15\x62ytes_to_write_failed\x18\x0c \x01(\x03\x12 \n\x18\x62ytes_to_write_completed\x18\r \x01(\x03\"\xa0\x03\n\x10\x43hannelStateInfo\x12;\n\x06groups\x18\x01 \x03(\x0b\x32+.ont.rpc.acquisition.ChannelStateInfo.Group\x1a;\n\x05Style\x12\r\n\x05label\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x0e\n\x06\x63olour\x18\x03 \x01(\t\x1az\n\x0c\x43hannelState\x12\n\n\x02id\x18\x01 \x01(\r\x12\x0c\n\x04name\x18\x02 \x01(\t\x12:\n\x05style\x18\x03 \x01(\x0b\x32+.ont.rpc.acquisition.ChannelStateInfo.Style\x12\x14\n\x0cglobal_order\x18\x04 \x01(\r\x1a\x95\x01\n\x05Group\x12\x0c\n\x04name\x18\x01 \x01(\t\x12:\n\x05style\x18\x02 \x01(\x0b\x32+.ont.rpc.acquisition.ChannelStateInfo.Style\x12\x42\n\x06states\x18\x03 \x03(\x0b\x32\x32.ont.rpc.acquisition.ChannelStateInfo.ChannelState\"\x8e\x03\n\x18\x41\x63quisitionConfigSummary\x12\x1b\n\x13\x62\x61secalling_enabled\x18\x01 \x01(\x08\x12\x17\n\x0freads_directory\x18\x02 \x01(\t\x12\"\n\x1areads_fallback_directories\x18\x03 \x03(\t\x12\x1b\n\x13\x66\x61st5_reads_enabled\x18\x04 \x01(\x08\x12\x1b\n\x13\x66\x61stq_reads_enabled\x18\x05 \x01(\x08\x12\x1e\n\x16protobuf_reads_enabled\x18\x06 \x01(\x08\x12\x16\n\x0e\x62ulk_file_path\x18\x07 \x01(\t\x12\x19\n\x11\x62ulk_file_enabled\x18\x08 \x01(\x08\x12\x41\n\x12\x63hannel_state_info\x18\t \x01(\x0b\x32%.ont.rpc.acquisition.ChannelStateInfo\x12\x1c\n\x14\x65vents_to_base_ratio\x18\n \x01(\x02\x12\x13\n\x0bsample_rate\x18\x0b \x01(\r\x12\x15\n\rchannel_count\x18\x0c \x01(\r\"\xb5\x04\n\x12\x41\x63quisitionRunInfo\x12\x0e\n\x06run_id\x18\x01 \x01(\t\x12\x34\n\x05state\x18\x02 \x01(\x0e\x32%.ont.rpc.acquisition.AcquisitionState\x12<\n\x0f\x66inishing_state\x18\n \x01(\x0e\x32#.ont.rpc.acquisition.FinishingState\x12?\n\x0bstop_reason\x18\x03 \x01(\x0e\x32*.ont.rpc.acquisition.AcquisitionStopReason\x12.\n\nstart_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x38\n\x14\x64\x61ta_read_start_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x36\n\x12\x64\x61ta_read_end_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08\x65nd_time\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x43\n\ryield_summary\x18\x08 \x01(\x0b\x32,.ont.rpc.acquisition.AcquisitionYieldSummary\x12\x45\n\x0e\x63onfig_summary\x18\t \x01(\x0b\x32-.ont.rpc.acquisition.AcquisitionConfigSummary\"\x1c\n\x1aListAcquisitionRunsRequest\".\n\x1bListAcquisitionRunsResponse\x12\x0f\n\x07run_ids\x18\x01 \x03(\t\"!\n\x1fGetCurrentAcquisitionRunRequest\"#\n!WatchCurrentAcquisitionRunRequest*Y\n\rMinknowStatus\x12\x10\n\x0c\x45RROR_STATUS\x10\x00\x12\t\n\x05READY\x10\x01\x12\x0c\n\x08STARTING\x10\x02\x12\x0e\n\nPROCESSING\x10\x03\x12\r\n\tFINISHING\x10\x04**\n\x06Option\x12\x08\n\x04\x41UTO\x10\x00\x12\x0b\n\x07\x44ISABLE\x10\x01\x12\t\n\x05\x46ORCE\x10\x02*=\n\x07Purpose\x12\x11\n\rOTHER_PURPOSE\x10\x00\x12\x0e\n\nSEQUENCING\x10\x02\x12\x0f\n\x0b\x43\x41LIBRATION\x10\x03*{\n\x10\x41\x63quisitionState\x12\x18\n\x14\x41\x43QUISITION_STARTING\x10\x00\x12\x17\n\x13\x41\x43QUISITION_RUNNING\x10\x01\x12\x19\n\x15\x41\x43QUISITION_FINISHING\x10\x02\x12\x19\n\x15\x41\x43QUISITION_COMPLETED\x10\x03*\xe3\x01\n\x15\x41\x63quisitionStopReason\x12\x13\n\x0fSTOPPED_NOT_SET\x10\x00\x12\x1a\n\x16STOPPED_USER_REQUESTED\x10\x01\x12\x19\n\x15STOPPED_NO_DISK_SPACE\x10\x02\x12&\n\"STOPPED_DEVICE_STOPPED_ACQUISITION\x10\x03\x12 \n\x1cSTOPPED_STARTING_ANOTHER_RUN\x10\x04\x12\x1a\n\x16STOPPED_PROTOCOL_ENDED\x10\x05\x12\x18\n\x14STOPPED_DEVICE_ERROR\x10\x06*\x8b\x01\n\x0e\x46inishingState\x12\x15\n\x11\x46INISHING_UNKNOWN\x10\x00\x12&\n\"FINISHING_PROCESSING_DEVICE_SIGNAL\x10\x01\x12\x1f\n\x1b\x46INISHING_BASECALLING_READS\x10\x02\x12\x19\n\x15\x46INISHING_SAVING_DATA\x10\x03\x32\x87\x08\n\x12\x41\x63quisitionService\x12P\n\x05start\x12!.ont.rpc.acquisition.StartRequest\x1a\".ont.rpc.acquisition.StartResponse\"\x00\x12M\n\x04stop\x12 .ont.rpc.acquisition.StopRequest\x1a!.ont.rpc.acquisition.StopResponse\"\x00\x12\x84\x01\n\x17watch_for_status_change\x12\x30.ont.rpc.acquisition.WatchForStatusChangeRequest\x1a\x31.ont.rpc.acquisition.WatchForStatusChangeResponse\"\x00(\x01\x30\x01\x12\x84\x01\n\x1dwatch_current_acquisition_run\x12\x36.ont.rpc.acquisition.WatchCurrentAcquisitionRunRequest\x1a\'.ont.rpc.acquisition.AcquisitionRunInfo\"\x00\x30\x01\x12i\n\x0e\x63urrent_status\x12).ont.rpc.acquisition.CurrentStatusRequest\x1a*.ont.rpc.acquisition.CurrentStatusResponse\"\x00\x12\x63\n\x0cget_progress\x12\'.ont.rpc.acquisition.GetProgressRequest\x1a(.ont.rpc.acquisition.GetProgressResponse\"\x00\x12t\n\x14get_acquisition_info\x12\x31.ont.rpc.acquisition.GetAcquisitionRunInfoRequest\x1a\'.ont.rpc.acquisition.AcquisitionRunInfo\"\x00\x12|\n\x15list_acquisition_runs\x12/.ont.rpc.acquisition.ListAcquisitionRunsRequest\x1a\x30.ont.rpc.acquisition.ListAcquisitionRunsResponse\"\x00\x12~\n\x1bget_current_acquisition_run\x12\x34.ont.rpc.acquisition.GetCurrentAcquisitionRunRequest\x1a\'.ont.rpc.acquisition.AcquisitionRunInfo\"\x00\x62\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])

_MINKNOWSTATUS = _descriptor.EnumDescriptor(
  name='MinknowStatus',
  full_name='ont.rpc.acquisition.MinknowStatus',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ERROR_STATUS', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='READY', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STARTING', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROCESSING', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FINISHING', index=4, number=4,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3196,
  serialized_end=3285,
)
_sym_db.RegisterEnumDescriptor(_MINKNOWSTATUS)

MinknowStatus = enum_type_wrapper.EnumTypeWrapper(_MINKNOWSTATUS)
_OPTION = _descriptor.EnumDescriptor(
  name='Option',
  full_name='ont.rpc.acquisition.Option',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='AUTO', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DISABLE', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FORCE', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3287,
  serialized_end=3329,
)
_sym_db.RegisterEnumDescriptor(_OPTION)

Option = enum_type_wrapper.EnumTypeWrapper(_OPTION)
_PURPOSE = _descriptor.EnumDescriptor(
  name='Purpose',
  full_name='ont.rpc.acquisition.Purpose',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OTHER_PURPOSE', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SEQUENCING', index=1, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CALIBRATION', index=2, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3331,
  serialized_end=3392,
)
_sym_db.RegisterEnumDescriptor(_PURPOSE)

Purpose = enum_type_wrapper.EnumTypeWrapper(_PURPOSE)
_ACQUISITIONSTATE = _descriptor.EnumDescriptor(
  name='AcquisitionState',
  full_name='ont.rpc.acquisition.AcquisitionState',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ACQUISITION_STARTING', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ACQUISITION_RUNNING', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ACQUISITION_FINISHING', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ACQUISITION_COMPLETED', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3394,
  serialized_end=3517,
)
_sym_db.RegisterEnumDescriptor(_ACQUISITIONSTATE)

AcquisitionState = enum_type_wrapper.EnumTypeWrapper(_ACQUISITIONSTATE)
_ACQUISITIONSTOPREASON = _descriptor.EnumDescriptor(
  name='AcquisitionStopReason',
  full_name='ont.rpc.acquisition.AcquisitionStopReason',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='STOPPED_NOT_SET', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STOPPED_USER_REQUESTED', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STOPPED_NO_DISK_SPACE', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STOPPED_DEVICE_STOPPED_ACQUISITION', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STOPPED_STARTING_ANOTHER_RUN', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STOPPED_PROTOCOL_ENDED', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STOPPED_DEVICE_ERROR', index=6, number=6,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3520,
  serialized_end=3747,
)
_sym_db.RegisterEnumDescriptor(_ACQUISITIONSTOPREASON)

AcquisitionStopReason = enum_type_wrapper.EnumTypeWrapper(_ACQUISITIONSTOPREASON)
_FINISHINGSTATE = _descriptor.EnumDescriptor(
  name='FinishingState',
  full_name='ont.rpc.acquisition.FinishingState',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='FINISHING_UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FINISHING_PROCESSING_DEVICE_SIGNAL', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FINISHING_BASECALLING_READS', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FINISHING_SAVING_DATA', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3750,
  serialized_end=3889,
)
_sym_db.RegisterEnumDescriptor(_FINISHINGSTATE)

FinishingState = enum_type_wrapper.EnumTypeWrapper(_FINISHINGSTATE)
ERROR_STATUS = 0
READY = 1
STARTING = 2
PROCESSING = 3
FINISHING = 4
AUTO = 0
DISABLE = 1
FORCE = 2
OTHER_PURPOSE = 0
SEQUENCING = 2
CALIBRATION = 3
ACQUISITION_STARTING = 0
ACQUISITION_RUNNING = 1
ACQUISITION_FINISHING = 2
ACQUISITION_COMPLETED = 3
STOPPED_NOT_SET = 0
STOPPED_USER_REQUESTED = 1
STOPPED_NO_DISK_SPACE = 2
STOPPED_DEVICE_STOPPED_ACQUISITION = 3
STOPPED_STARTING_ANOTHER_RUN = 4
STOPPED_PROTOCOL_ENDED = 5
STOPPED_DEVICE_ERROR = 6
FINISHING_UNKNOWN = 0
FINISHING_PROCESSING_DEVICE_SIGNAL = 1
FINISHING_BASECALLING_READS = 2
FINISHING_SAVING_DATA = 3


_STOPREQUEST_DATAACTION = _descriptor.EnumDescriptor(
  name='DataAction',
  full_name='ont.rpc.acquisition.StopRequest.DataAction',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='STOP_DEFAULT', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STOP_KEEP_ALL_DATA', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STOP_FINISH_PROCESSING', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=671,
  serialized_end=753,
)
_sym_db.RegisterEnumDescriptor(_STOPREQUEST_DATAACTION)


_STARTREQUEST = _descriptor.Descriptor(
  name='StartRequest',
  full_name='ont.rpc.acquisition.StartRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='wait_until_processing', full_name='ont.rpc.acquisition.StartRequest.wait_until_processing', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dont_wait_for_device_ready', full_name='ont.rpc.acquisition.StartRequest.dont_wait_for_device_ready', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='generate_report', full_name='ont.rpc.acquisition.StartRequest.generate_report', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='send_sequencing_read_metrics', full_name='ont.rpc.acquisition.StartRequest.send_sequencing_read_metrics', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='send_basecalling_metrics', full_name='ont.rpc.acquisition.StartRequest.send_basecalling_metrics', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='purpose', full_name='ont.rpc.acquisition.StartRequest.purpose', index=5,
      number=6, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='analysis', full_name='ont.rpc.acquisition.StartRequest.analysis', index=6,
      number=7, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='file_output', full_name='ont.rpc.acquisition.StartRequest.file_output', index=7,
      number=8, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=88,
  serialized_end=497,
)


_STARTRESPONSE = _descriptor.Descriptor(
  name='StartResponse',
  full_name='ont.rpc.acquisition.StartResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='run_id', full_name='ont.rpc.acquisition.StartResponse.run_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=499,
  serialized_end=530,
)


_STOPREQUEST = _descriptor.Descriptor(
  name='StopRequest',
  full_name='ont.rpc.acquisition.StopRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data_action_on_stop', full_name='ont.rpc.acquisition.StopRequest.data_action_on_stop', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='wait_until_ready', full_name='ont.rpc.acquisition.StopRequest.wait_until_ready', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='keep_power_on', full_name='ont.rpc.acquisition.StopRequest.keep_power_on', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _STOPREQUEST_DATAACTION,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=533,
  serialized_end=753,
)


_STOPRESPONSE = _descriptor.Descriptor(
  name='StopResponse',
  full_name='ont.rpc.acquisition.StopResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=755,
  serialized_end=769,
)


_WATCHFORSTATUSCHANGEREQUEST = _descriptor.Descriptor(
  name='WatchForStatusChangeRequest',
  full_name='ont.rpc.acquisition.WatchForStatusChangeRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stop', full_name='ont.rpc.acquisition.WatchForStatusChangeRequest.stop', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=771,
  serialized_end=814,
)


_WATCHFORSTATUSCHANGERESPONSE = _descriptor.Descriptor(
  name='WatchForStatusChangeResponse',
  full_name='ont.rpc.acquisition.WatchForStatusChangeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='ont.rpc.acquisition.WatchForStatusChangeResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=816,
  serialized_end=898,
)


_CURRENTSTATUSREQUEST = _descriptor.Descriptor(
  name='CurrentStatusRequest',
  full_name='ont.rpc.acquisition.CurrentStatusRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=900,
  serialized_end=922,
)


_CURRENTSTATUSRESPONSE = _descriptor.Descriptor(
  name='CurrentStatusResponse',
  full_name='ont.rpc.acquisition.CurrentStatusResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='ont.rpc.acquisition.CurrentStatusResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=924,
  serialized_end=999,
)


_GETPROGRESSREQUEST = _descriptor.Descriptor(
  name='GetProgressRequest',
  full_name='ont.rpc.acquisition.GetProgressRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1001,
  serialized_end=1021,
)


_GETPROGRESSRESPONSE_RAWPERCHANNEL = _descriptor.Descriptor(
  name='RawPerChannel',
  full_name='ont.rpc.acquisition.GetProgressResponse.RawPerChannel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='acquired', full_name='ont.rpc.acquisition.GetProgressResponse.RawPerChannel.acquired', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='processed', full_name='ont.rpc.acquisition.GetProgressResponse.RawPerChannel.processed', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1128,
  serialized_end=1180,
)

_GETPROGRESSRESPONSE = _descriptor.Descriptor(
  name='GetProgressResponse',
  full_name='ont.rpc.acquisition.GetProgressResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='raw_per_channel', full_name='ont.rpc.acquisition.GetProgressResponse.raw_per_channel', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETPROGRESSRESPONSE_RAWPERCHANNEL, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1024,
  serialized_end=1180,
)


_GETACQUISITIONRUNINFOREQUEST = _descriptor.Descriptor(
  name='GetAcquisitionRunInfoRequest',
  full_name='ont.rpc.acquisition.GetAcquisitionRunInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='run_id', full_name='ont.rpc.acquisition.GetAcquisitionRunInfoRequest.run_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1182,
  serialized_end=1228,
)


_ACQUISITIONYIELDSUMMARY = _descriptor.Descriptor(
  name='AcquisitionYieldSummary',
  full_name='ont.rpc.acquisition.AcquisitionYieldSummary',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='read_count', full_name='ont.rpc.acquisition.AcquisitionYieldSummary.read_count', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='written_read_count', full_name='ont.rpc.acquisition.AcquisitionYieldSummary.written_read_count', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='basecalled_pass_read_count', full_name='ont.rpc.acquisition.AcquisitionYieldSummary.basecalled_pass_read_count', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='basecalled_fail_read_count', full_name='ont.rpc.acquisition.AcquisitionYieldSummary.basecalled_fail_read_count', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='basecalled_skipped_read_count', full_name='ont.rpc.acquisition.AcquisitionYieldSummary.basecalled_skipped_read_count', index=4,
      number=10, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='basecalled_bases', full_name='ont.rpc.acquisition.AcquisitionYieldSummary.basecalled_bases', index=5,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='basecalled_samples', full_name='ont.rpc.acquisition.AcquisitionYieldSummary.basecalled_samples', index=6,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='selected_raw_samples', full_name='ont.rpc.acquisition.AcquisitionYieldSummary.selected_raw_samples', index=7,
      number=7, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='selected_events', full_name='ont.rpc.acquisition.AcquisitionYieldSummary.selected_events', index=8,
      number=8, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='estimated_selected_bases', full_name='ont.rpc.acquisition.AcquisitionYieldSummary.estimated_selected_bases', index=9,
      number=9, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bytes_to_write_produced', full_name='ont.rpc.acquisition.AcquisitionYieldSummary.bytes_to_write_produced', index=10,
      number=11, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bytes_to_write_failed', full_name='ont.rpc.acquisition.AcquisitionYieldSummary.bytes_to_write_failed', index=11,
      number=12, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bytes_to_write_completed', full_name='ont.rpc.acquisition.AcquisitionYieldSummary.bytes_to_write_completed', index=12,
      number=13, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1231,
  serialized_end=1656,
)


_CHANNELSTATEINFO_STYLE = _descriptor.Descriptor(
  name='Style',
  full_name='ont.rpc.acquisition.ChannelStateInfo.Style',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='label', full_name='ont.rpc.acquisition.ChannelStateInfo.Style.label', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='ont.rpc.acquisition.ChannelStateInfo.Style.description', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='colour', full_name='ont.rpc.acquisition.ChannelStateInfo.Style.colour', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1740,
  serialized_end=1799,
)

_CHANNELSTATEINFO_CHANNELSTATE = _descriptor.Descriptor(
  name='ChannelState',
  full_name='ont.rpc.acquisition.ChannelStateInfo.ChannelState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='ont.rpc.acquisition.ChannelStateInfo.ChannelState.id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='ont.rpc.acquisition.ChannelStateInfo.ChannelState.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='style', full_name='ont.rpc.acquisition.ChannelStateInfo.ChannelState.style', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='global_order', full_name='ont.rpc.acquisition.ChannelStateInfo.ChannelState.global_order', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1801,
  serialized_end=1923,
)

_CHANNELSTATEINFO_GROUP = _descriptor.Descriptor(
  name='Group',
  full_name='ont.rpc.acquisition.ChannelStateInfo.Group',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='ont.rpc.acquisition.ChannelStateInfo.Group.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='style', full_name='ont.rpc.acquisition.ChannelStateInfo.Group.style', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='states', full_name='ont.rpc.acquisition.ChannelStateInfo.Group.states', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1926,
  serialized_end=2075,
)

_CHANNELSTATEINFO = _descriptor.Descriptor(
  name='ChannelStateInfo',
  full_name='ont.rpc.acquisition.ChannelStateInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='groups', full_name='ont.rpc.acquisition.ChannelStateInfo.groups', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CHANNELSTATEINFO_STYLE, _CHANNELSTATEINFO_CHANNELSTATE, _CHANNELSTATEINFO_GROUP, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1659,
  serialized_end=2075,
)


_ACQUISITIONCONFIGSUMMARY = _descriptor.Descriptor(
  name='AcquisitionConfigSummary',
  full_name='ont.rpc.acquisition.AcquisitionConfigSummary',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='basecalling_enabled', full_name='ont.rpc.acquisition.AcquisitionConfigSummary.basecalling_enabled', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='reads_directory', full_name='ont.rpc.acquisition.AcquisitionConfigSummary.reads_directory', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='reads_fallback_directories', full_name='ont.rpc.acquisition.AcquisitionConfigSummary.reads_fallback_directories', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fast5_reads_enabled', full_name='ont.rpc.acquisition.AcquisitionConfigSummary.fast5_reads_enabled', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fastq_reads_enabled', full_name='ont.rpc.acquisition.AcquisitionConfigSummary.fastq_reads_enabled', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='protobuf_reads_enabled', full_name='ont.rpc.acquisition.AcquisitionConfigSummary.protobuf_reads_enabled', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bulk_file_path', full_name='ont.rpc.acquisition.AcquisitionConfigSummary.bulk_file_path', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bulk_file_enabled', full_name='ont.rpc.acquisition.AcquisitionConfigSummary.bulk_file_enabled', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='channel_state_info', full_name='ont.rpc.acquisition.AcquisitionConfigSummary.channel_state_info', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='events_to_base_ratio', full_name='ont.rpc.acquisition.AcquisitionConfigSummary.events_to_base_ratio', index=9,
      number=10, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sample_rate', full_name='ont.rpc.acquisition.AcquisitionConfigSummary.sample_rate', index=10,
      number=11, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='channel_count', full_name='ont.rpc.acquisition.AcquisitionConfigSummary.channel_count', index=11,
      number=12, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2078,
  serialized_end=2476,
)


_ACQUISITIONRUNINFO = _descriptor.Descriptor(
  name='AcquisitionRunInfo',
  full_name='ont.rpc.acquisition.AcquisitionRunInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='run_id', full_name='ont.rpc.acquisition.AcquisitionRunInfo.run_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='state', full_name='ont.rpc.acquisition.AcquisitionRunInfo.state', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='finishing_state', full_name='ont.rpc.acquisition.AcquisitionRunInfo.finishing_state', index=2,
      number=10, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stop_reason', full_name='ont.rpc.acquisition.AcquisitionRunInfo.stop_reason', index=3,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start_time', full_name='ont.rpc.acquisition.AcquisitionRunInfo.start_time', index=4,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data_read_start_time', full_name='ont.rpc.acquisition.AcquisitionRunInfo.data_read_start_time', index=5,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data_read_end_time', full_name='ont.rpc.acquisition.AcquisitionRunInfo.data_read_end_time', index=6,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='end_time', full_name='ont.rpc.acquisition.AcquisitionRunInfo.end_time', index=7,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='yield_summary', full_name='ont.rpc.acquisition.AcquisitionRunInfo.yield_summary', index=8,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='config_summary', full_name='ont.rpc.acquisition.AcquisitionRunInfo.config_summary', index=9,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2479,
  serialized_end=3044,
)


_LISTACQUISITIONRUNSREQUEST = _descriptor.Descriptor(
  name='ListAcquisitionRunsRequest',
  full_name='ont.rpc.acquisition.ListAcquisitionRunsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3046,
  serialized_end=3074,
)


_LISTACQUISITIONRUNSRESPONSE = _descriptor.Descriptor(
  name='ListAcquisitionRunsResponse',
  full_name='ont.rpc.acquisition.ListAcquisitionRunsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='run_ids', full_name='ont.rpc.acquisition.ListAcquisitionRunsResponse.run_ids', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3076,
  serialized_end=3122,
)


_GETCURRENTACQUISITIONRUNREQUEST = _descriptor.Descriptor(
  name='GetCurrentAcquisitionRunRequest',
  full_name='ont.rpc.acquisition.GetCurrentAcquisitionRunRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3124,
  serialized_end=3157,
)


_WATCHCURRENTACQUISITIONRUNREQUEST = _descriptor.Descriptor(
  name='WatchCurrentAcquisitionRunRequest',
  full_name='ont.rpc.acquisition.WatchCurrentAcquisitionRunRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3159,
  serialized_end=3194,
)

_STARTREQUEST.fields_by_name['generate_report'].enum_type = _OPTION
_STARTREQUEST.fields_by_name['send_sequencing_read_metrics'].enum_type = _OPTION
_STARTREQUEST.fields_by_name['send_basecalling_metrics'].enum_type = _OPTION
_STARTREQUEST.fields_by_name['purpose'].enum_type = _PURPOSE
_STARTREQUEST.fields_by_name['analysis'].enum_type = _OPTION
_STARTREQUEST.fields_by_name['file_output'].enum_type = _OPTION
_STOPREQUEST.fields_by_name['data_action_on_stop'].enum_type = _STOPREQUEST_DATAACTION
_STOPREQUEST_DATAACTION.containing_type = _STOPREQUEST
_WATCHFORSTATUSCHANGERESPONSE.fields_by_name['status'].enum_type = _MINKNOWSTATUS
_CURRENTSTATUSRESPONSE.fields_by_name['status'].enum_type = _MINKNOWSTATUS
_GETPROGRESSRESPONSE_RAWPERCHANNEL.containing_type = _GETPROGRESSRESPONSE
_GETPROGRESSRESPONSE.fields_by_name['raw_per_channel'].message_type = _GETPROGRESSRESPONSE_RAWPERCHANNEL
_CHANNELSTATEINFO_STYLE.containing_type = _CHANNELSTATEINFO
_CHANNELSTATEINFO_CHANNELSTATE.fields_by_name['style'].message_type = _CHANNELSTATEINFO_STYLE
_CHANNELSTATEINFO_CHANNELSTATE.containing_type = _CHANNELSTATEINFO
_CHANNELSTATEINFO_GROUP.fields_by_name['style'].message_type = _CHANNELSTATEINFO_STYLE
_CHANNELSTATEINFO_GROUP.fields_by_name['states'].message_type = _CHANNELSTATEINFO_CHANNELSTATE
_CHANNELSTATEINFO_GROUP.containing_type = _CHANNELSTATEINFO
_CHANNELSTATEINFO.fields_by_name['groups'].message_type = _CHANNELSTATEINFO_GROUP
_ACQUISITIONCONFIGSUMMARY.fields_by_name['channel_state_info'].message_type = _CHANNELSTATEINFO
_ACQUISITIONRUNINFO.fields_by_name['state'].enum_type = _ACQUISITIONSTATE
_ACQUISITIONRUNINFO.fields_by_name['finishing_state'].enum_type = _FINISHINGSTATE
_ACQUISITIONRUNINFO.fields_by_name['stop_reason'].enum_type = _ACQUISITIONSTOPREASON
_ACQUISITIONRUNINFO.fields_by_name['start_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ACQUISITIONRUNINFO.fields_by_name['data_read_start_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ACQUISITIONRUNINFO.fields_by_name['data_read_end_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ACQUISITIONRUNINFO.fields_by_name['end_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ACQUISITIONRUNINFO.fields_by_name['yield_summary'].message_type = _ACQUISITIONYIELDSUMMARY
_ACQUISITIONRUNINFO.fields_by_name['config_summary'].message_type = _ACQUISITIONCONFIGSUMMARY
DESCRIPTOR.message_types_by_name['StartRequest'] = _STARTREQUEST
DESCRIPTOR.message_types_by_name['StartResponse'] = _STARTRESPONSE
DESCRIPTOR.message_types_by_name['StopRequest'] = _STOPREQUEST
DESCRIPTOR.message_types_by_name['StopResponse'] = _STOPRESPONSE
DESCRIPTOR.message_types_by_name['WatchForStatusChangeRequest'] = _WATCHFORSTATUSCHANGEREQUEST
DESCRIPTOR.message_types_by_name['WatchForStatusChangeResponse'] = _WATCHFORSTATUSCHANGERESPONSE
DESCRIPTOR.message_types_by_name['CurrentStatusRequest'] = _CURRENTSTATUSREQUEST
DESCRIPTOR.message_types_by_name['CurrentStatusResponse'] = _CURRENTSTATUSRESPONSE
DESCRIPTOR.message_types_by_name['GetProgressRequest'] = _GETPROGRESSREQUEST
DESCRIPTOR.message_types_by_name['GetProgressResponse'] = _GETPROGRESSRESPONSE
DESCRIPTOR.message_types_by_name['GetAcquisitionRunInfoRequest'] = _GETACQUISITIONRUNINFOREQUEST
DESCRIPTOR.message_types_by_name['AcquisitionYieldSummary'] = _ACQUISITIONYIELDSUMMARY
DESCRIPTOR.message_types_by_name['ChannelStateInfo'] = _CHANNELSTATEINFO
DESCRIPTOR.message_types_by_name['AcquisitionConfigSummary'] = _ACQUISITIONCONFIGSUMMARY
DESCRIPTOR.message_types_by_name['AcquisitionRunInfo'] = _ACQUISITIONRUNINFO
DESCRIPTOR.message_types_by_name['ListAcquisitionRunsRequest'] = _LISTACQUISITIONRUNSREQUEST
DESCRIPTOR.message_types_by_name['ListAcquisitionRunsResponse'] = _LISTACQUISITIONRUNSRESPONSE
DESCRIPTOR.message_types_by_name['GetCurrentAcquisitionRunRequest'] = _GETCURRENTACQUISITIONRUNREQUEST
DESCRIPTOR.message_types_by_name['WatchCurrentAcquisitionRunRequest'] = _WATCHCURRENTACQUISITIONRUNREQUEST
DESCRIPTOR.enum_types_by_name['MinknowStatus'] = _MINKNOWSTATUS
DESCRIPTOR.enum_types_by_name['Option'] = _OPTION
DESCRIPTOR.enum_types_by_name['Purpose'] = _PURPOSE
DESCRIPTOR.enum_types_by_name['AcquisitionState'] = _ACQUISITIONSTATE
DESCRIPTOR.enum_types_by_name['AcquisitionStopReason'] = _ACQUISITIONSTOPREASON
DESCRIPTOR.enum_types_by_name['FinishingState'] = _FINISHINGSTATE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

StartRequest = _reflection.GeneratedProtocolMessageType('StartRequest', (_message.Message,), {
  'DESCRIPTOR' : _STARTREQUEST,
  '__module__' : 'minknow.rpc.acquisition_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.StartRequest)
  })
_sym_db.RegisterMessage(StartRequest)

StartResponse = _reflection.GeneratedProtocolMessageType('StartResponse', (_message.Message,), {
  'DESCRIPTOR' : _STARTRESPONSE,
  '__module__' : 'minknow.rpc.acquisition_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.StartResponse)
  })
_sym_db.RegisterMessage(StartResponse)

StopRequest = _reflection.GeneratedProtocolMessageType('StopRequest', (_message.Message,), {
  'DESCRIPTOR' : _STOPREQUEST,
  '__module__' : 'minknow.rpc.acquisition_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.StopRequest)
  })
_sym_db.RegisterMessage(StopRequest)

StopResponse = _reflection.GeneratedProtocolMessageType('StopResponse', (_message.Message,), {
  'DESCRIPTOR' : _STOPRESPONSE,
  '__module__' : 'minknow.rpc.acquisition_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.StopResponse)
  })
_sym_db.RegisterMessage(StopResponse)

WatchForStatusChangeRequest = _reflection.GeneratedProtocolMessageType('WatchForStatusChangeRequest', (_message.Message,), {
  'DESCRIPTOR' : _WATCHFORSTATUSCHANGEREQUEST,
  '__module__' : 'minknow.rpc.acquisition_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.WatchForStatusChangeRequest)
  })
_sym_db.RegisterMessage(WatchForStatusChangeRequest)

WatchForStatusChangeResponse = _reflection.GeneratedProtocolMessageType('WatchForStatusChangeResponse', (_message.Message,), {
  'DESCRIPTOR' : _WATCHFORSTATUSCHANGERESPONSE,
  '__module__' : 'minknow.rpc.acquisition_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.WatchForStatusChangeResponse)
  })
_sym_db.RegisterMessage(WatchForStatusChangeResponse)

CurrentStatusRequest = _reflection.GeneratedProtocolMessageType('CurrentStatusRequest', (_message.Message,), {
  'DESCRIPTOR' : _CURRENTSTATUSREQUEST,
  '__module__' : 'minknow.rpc.acquisition_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.CurrentStatusRequest)
  })
_sym_db.RegisterMessage(CurrentStatusRequest)

CurrentStatusResponse = _reflection.GeneratedProtocolMessageType('CurrentStatusResponse', (_message.Message,), {
  'DESCRIPTOR' : _CURRENTSTATUSRESPONSE,
  '__module__' : 'minknow.rpc.acquisition_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.CurrentStatusResponse)
  })
_sym_db.RegisterMessage(CurrentStatusResponse)

GetProgressRequest = _reflection.GeneratedProtocolMessageType('GetProgressRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETPROGRESSREQUEST,
  '__module__' : 'minknow.rpc.acquisition_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.GetProgressRequest)
  })
_sym_db.RegisterMessage(GetProgressRequest)

GetProgressResponse = _reflection.GeneratedProtocolMessageType('GetProgressResponse', (_message.Message,), {

  'RawPerChannel' : _reflection.GeneratedProtocolMessageType('RawPerChannel', (_message.Message,), {
    'DESCRIPTOR' : _GETPROGRESSRESPONSE_RAWPERCHANNEL,
    '__module__' : 'minknow.rpc.acquisition_pb2'
    # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.GetProgressResponse.RawPerChannel)
    })
  ,
  'DESCRIPTOR' : _GETPROGRESSRESPONSE,
  '__module__' : 'minknow.rpc.acquisition_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.GetProgressResponse)
  })
_sym_db.RegisterMessage(GetProgressResponse)
_sym_db.RegisterMessage(GetProgressResponse.RawPerChannel)

GetAcquisitionRunInfoRequest = _reflection.GeneratedProtocolMessageType('GetAcquisitionRunInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETACQUISITIONRUNINFOREQUEST,
  '__module__' : 'minknow.rpc.acquisition_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.GetAcquisitionRunInfoRequest)
  })
_sym_db.RegisterMessage(GetAcquisitionRunInfoRequest)

AcquisitionYieldSummary = _reflection.GeneratedProtocolMessageType('AcquisitionYieldSummary', (_message.Message,), {
  'DESCRIPTOR' : _ACQUISITIONYIELDSUMMARY,
  '__module__' : 'minknow.rpc.acquisition_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.AcquisitionYieldSummary)
  })
_sym_db.RegisterMessage(AcquisitionYieldSummary)

ChannelStateInfo = _reflection.GeneratedProtocolMessageType('ChannelStateInfo', (_message.Message,), {

  'Style' : _reflection.GeneratedProtocolMessageType('Style', (_message.Message,), {
    'DESCRIPTOR' : _CHANNELSTATEINFO_STYLE,
    '__module__' : 'minknow.rpc.acquisition_pb2'
    # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.ChannelStateInfo.Style)
    })
  ,

  'ChannelState' : _reflection.GeneratedProtocolMessageType('ChannelState', (_message.Message,), {
    'DESCRIPTOR' : _CHANNELSTATEINFO_CHANNELSTATE,
    '__module__' : 'minknow.rpc.acquisition_pb2'
    # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.ChannelStateInfo.ChannelState)
    })
  ,

  'Group' : _reflection.GeneratedProtocolMessageType('Group', (_message.Message,), {
    'DESCRIPTOR' : _CHANNELSTATEINFO_GROUP,
    '__module__' : 'minknow.rpc.acquisition_pb2'
    # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.ChannelStateInfo.Group)
    })
  ,
  'DESCRIPTOR' : _CHANNELSTATEINFO,
  '__module__' : 'minknow.rpc.acquisition_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.ChannelStateInfo)
  })
_sym_db.RegisterMessage(ChannelStateInfo)
_sym_db.RegisterMessage(ChannelStateInfo.Style)
_sym_db.RegisterMessage(ChannelStateInfo.ChannelState)
_sym_db.RegisterMessage(ChannelStateInfo.Group)

AcquisitionConfigSummary = _reflection.GeneratedProtocolMessageType('AcquisitionConfigSummary', (_message.Message,), {
  'DESCRIPTOR' : _ACQUISITIONCONFIGSUMMARY,
  '__module__' : 'minknow.rpc.acquisition_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.AcquisitionConfigSummary)
  })
_sym_db.RegisterMessage(AcquisitionConfigSummary)

AcquisitionRunInfo = _reflection.GeneratedProtocolMessageType('AcquisitionRunInfo', (_message.Message,), {
  'DESCRIPTOR' : _ACQUISITIONRUNINFO,
  '__module__' : 'minknow.rpc.acquisition_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.AcquisitionRunInfo)
  })
_sym_db.RegisterMessage(AcquisitionRunInfo)

ListAcquisitionRunsRequest = _reflection.GeneratedProtocolMessageType('ListAcquisitionRunsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTACQUISITIONRUNSREQUEST,
  '__module__' : 'minknow.rpc.acquisition_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.ListAcquisitionRunsRequest)
  })
_sym_db.RegisterMessage(ListAcquisitionRunsRequest)

ListAcquisitionRunsResponse = _reflection.GeneratedProtocolMessageType('ListAcquisitionRunsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTACQUISITIONRUNSRESPONSE,
  '__module__' : 'minknow.rpc.acquisition_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.ListAcquisitionRunsResponse)
  })
_sym_db.RegisterMessage(ListAcquisitionRunsResponse)

GetCurrentAcquisitionRunRequest = _reflection.GeneratedProtocolMessageType('GetCurrentAcquisitionRunRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETCURRENTACQUISITIONRUNREQUEST,
  '__module__' : 'minknow.rpc.acquisition_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.GetCurrentAcquisitionRunRequest)
  })
_sym_db.RegisterMessage(GetCurrentAcquisitionRunRequest)

WatchCurrentAcquisitionRunRequest = _reflection.GeneratedProtocolMessageType('WatchCurrentAcquisitionRunRequest', (_message.Message,), {
  'DESCRIPTOR' : _WATCHCURRENTACQUISITIONRUNREQUEST,
  '__module__' : 'minknow.rpc.acquisition_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.acquisition.WatchCurrentAcquisitionRunRequest)
  })
_sym_db.RegisterMessage(WatchCurrentAcquisitionRunRequest)



_ACQUISITIONSERVICE = _descriptor.ServiceDescriptor(
  name='AcquisitionService',
  full_name='ont.rpc.acquisition.AcquisitionService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=3892,
  serialized_end=4923,
  methods=[
  _descriptor.MethodDescriptor(
    name='start',
    full_name='ont.rpc.acquisition.AcquisitionService.start',
    index=0,
    containing_service=None,
    input_type=_STARTREQUEST,
    output_type=_STARTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='stop',
    full_name='ont.rpc.acquisition.AcquisitionService.stop',
    index=1,
    containing_service=None,
    input_type=_STOPREQUEST,
    output_type=_STOPRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='watch_for_status_change',
    full_name='ont.rpc.acquisition.AcquisitionService.watch_for_status_change',
    index=2,
    containing_service=None,
    input_type=_WATCHFORSTATUSCHANGEREQUEST,
    output_type=_WATCHFORSTATUSCHANGERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='watch_current_acquisition_run',
    full_name='ont.rpc.acquisition.AcquisitionService.watch_current_acquisition_run',
    index=3,
    containing_service=None,
    input_type=_WATCHCURRENTACQUISITIONRUNREQUEST,
    output_type=_ACQUISITIONRUNINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='current_status',
    full_name='ont.rpc.acquisition.AcquisitionService.current_status',
    index=4,
    containing_service=None,
    input_type=_CURRENTSTATUSREQUEST,
    output_type=_CURRENTSTATUSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_progress',
    full_name='ont.rpc.acquisition.AcquisitionService.get_progress',
    index=5,
    containing_service=None,
    input_type=_GETPROGRESSREQUEST,
    output_type=_GETPROGRESSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_acquisition_info',
    full_name='ont.rpc.acquisition.AcquisitionService.get_acquisition_info',
    index=6,
    containing_service=None,
    input_type=_GETACQUISITIONRUNINFOREQUEST,
    output_type=_ACQUISITIONRUNINFO,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='list_acquisition_runs',
    full_name='ont.rpc.acquisition.AcquisitionService.list_acquisition_runs',
    index=7,
    containing_service=None,
    input_type=_LISTACQUISITIONRUNSREQUEST,
    output_type=_LISTACQUISITIONRUNSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_current_acquisition_run',
    full_name='ont.rpc.acquisition.AcquisitionService.get_current_acquisition_run',
    index=8,
    containing_service=None,
    input_type=_GETCURRENTACQUISITIONRUNREQUEST,
    output_type=_ACQUISITIONRUNINFO,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_ACQUISITIONSERVICE)

DESCRIPTOR.services_by_name['AcquisitionService'] = _ACQUISITIONSERVICE

# @@protoc_insertion_point(module_scope)



import peewee
from peewee import *
#from porerefiner.config import config

import asyncio
import datetime
import logging
import pathlib
import pickle
import namesgenerator
import sys
import tempfile

from copy import copy, deepcopy
from itertools import chain

logging.addLevelName(logging.DEBUG - 5, 'TRACE')

#slightly demote the Peewee debug logging
peewee.logger.debug = lambda msg, *a, **k: peewee.logger.log(logging.DEBUG - 5, msg, *a, **k)


_db = SqliteDatabase(None)

class BaseModel(Model):

    class Meta:
        database = _db

class PorerefinerModel(BaseModel):
    "Abstract base class for PoreRefiner models"

    statuses = [('READY', 'Ready to Run'),
                ('QUEUED', 'Scheduled to Run'),
                ('RUNNING', 'Running'),
                ('STOPPING', 'Stopping'),
                ('DONE', 'Ended'),
                ('FAILED', 'Ended with Failure')]

    @property
    def tags(self):
        # query = Tag.select().join(TagJunction).join(Tag)
        # return [tag_j.tag for tag_j in self.tag_junctions]
        return (Tag.select()
                   .join(TagJunction)
                   .join(type(self))
                   .where(type(self).pk == self.pk))

    # def tag(self, *tags):
    #     for tag in tags:
    #         Tag.get_or_create(name=tag)



class Tag(BaseModel):
    "A tag is an informal annotation"

    name = CharField(null=False, constraints=[Check("name != '' ")])

    def __str__(self):
        return self.name

class PathField(Field):

    field_type = 'varchar'

    def db_value(self, value):
        if hasattr(value, '__fspath__'):
            value = str(value)
        if isinstance(value, bytes):
            value = str(value.decode(sys.getfilesystemencoding(), 'surrogateescape'))
        return value

    def python_value(self, value):
        if value:
            return pathlib.Path(value)
        return None


# class JobField(Field):

#     field_type = 'blob'

#     def db_value(self, value):
#         from porerefiner.jobs import AbstractJob
#         if not isinstance(value, AbstractJob):
#             raise ValueError(f"value of type {type(value)} can't be stored in this field.")
#         return pickle.dumps(value)

#     def python_value(self, value):
#         return pickle.loads(value)

def StatusField(*args, default=PorerefinerModel.statuses[0][0], **kwargs):
    return CharField(*args, choices=PorerefinerModel.statuses, default=default, **kwargs)


def create_readable_name():
    "Docker-style random name from namesgenerator"
    return namesgenerator.get_random_name()

# class Flowcell(PorerefinerModel):
#     "A flowcell is the disposable part of the sequencer"

#     pk = AutoField()

#     consumable_id = CharField(null=False)
#     consumable_name = CharField(null=False)
#     consumable_type = CharField(null=True)
#     path = PathField(index=True)

class Run(PorerefinerModel):
    "A run is an annotated collection of files being produced"

    basecallers = [('DNA', 'DNA Basecaller'),
                   ('RNA', 'RNA Basecaller')]

    pk = AutoField()

    # flowcell = ForeignKeyField(Flowcell, backref='runs')
    flowcell = CharField(null=True)
    _sample_sheet = DeferredForeignKey('SampleSheet', null=True, backref='runs')

    name = CharField()
    library_id = CharField(null=True)
    alt_name = CharField(default = create_readable_name)
    run_id = CharField(null=True)
    started = DateTimeField(default = datetime.datetime.now)
    ended = DateTimeField(null=True, default=None)
    status = StatusField(default='RUNNING')
    path = PathField(index=True)
    basecalling_model = CharField(default='DNA', choices=basecallers, null=True)

    def __str__(self):
        return f"{self.pk} {self.alt_name} ({self.path}) ({dict(self.statuses)[self.status]})"

    @property
    def all_files(self):
        return chain(self.files, chain(*(sample.files for sample in self.samples)))

    @property
    def jobs(self):
        return chain(*(file.jobs for file in self.all_files))

    @property
    def run_duration(self):
        if self.ended:
            return self.ended - self.started

    @property
    def sample_sheet(self):
        return self._sample_sheet or SampleSheet()

    @sample_sheet.setter
    def sample_sheet(self, ss):
        self._sample_sheet = ss

    @property
    def samples(self):
        return self.sample_sheet.samples

    @classmethod
    def get_unannotated_runs(cls):
        return cls.select().where(cls._sample_sheet.is_null(), cls.status=='RUNNING')

    def tag(self, tag):
        ta, _ = Tag.get_or_create(name=tag)
        tj, _ = TagJunction.get_or_create(tag=ta, run=self)
        return ta

    def untag(self, tag):
        ta = Tag.get_or_none(name=tag)
        if ta:
            TagJunction.delete().where(TagJunction.run==self, TagJunction.tag==ta).execute()

    @property
    def tags(self):
        return (Tag.select()
                   .join(TagJunction)
                   .where(TagJunction.run == self))

    def spawn(self, job_config):
        job = Job.create(status='READY', job_class=job_config.__class__.__name__, datadir=pathlib.Path(tempfile.mkdtemp()), run=self)
        # JobRunJunction.create(job=job, run=self)
        # for file in self.files:
        #     JobFileJunction.create(job=job, file=file)
        return job

# class JobRunJunction(BaseModel):
#     pk = AutoField()
#     job = DeferredForeignKey('Job', backref='jobs')
#     run = ForeignKeyField(Run, backref='runs')


class Qa(PorerefinerModel):
    "A QA is a set of quality-control analysis metrics"

    pk = AutoField()
    coverage = FloatField()
    quality = FloatField()


class Job(PorerefinerModel):
    "A job is a scheduled HPC job, pre or post submission"
    pk = AutoField()
    job_id = CharField(null=True)
    # job_state = JobField(null=True)
    job_class = TextField(null=False)
    status = StatusField(default='QUEUED')
    datadir = PathField(null=False)
    outputdir = PathField(null=True)
    run = ForeignKeyField(Run, null=True, backref='jobs')
    file = DeferredForeignKey('File', backref='_jobs_with_this_file_as_primary', null=True)


    def __str__(self):
        return f"{self.pk} ({self.job_class}) ({dict(self.statuses)[self.status]})"

    # @property
    # def files(self):
    #     return (File.select()
    #                 .join(JobFileJunction)
    #                 .join(Job)
    #                 .where(Job.pk == self.pk))

    @property
    def job_state(self):
        import porerefiner.jobs
        return porerefiner.jobs.CONFIGURED_JOB_REGISTRY[self.job_class]

    def tag(self, tag):
        ta, _ = Tag.get_or_create(name=tag)
        tj, _ = TagJunction.get_or_create(tag=ta, job=self)
        return ta

# class JobFileJunction(BaseModel):
#     pk = AutoField()
#     job = ForeignKeyField(Job, backref='jobs')
#     file = DeferredForeignKey('File', backref='files')

class SampleSheet(PorerefinerModel):
    "A samplesheet is a particular file, eventually attached to a run"

    BARCODES = [('SQK-16S024','16S Barcoding Kit 1-24 SQK-16S024'),
                ('EXP-NBD104','Native Barcoding Expansion 1-12 EXP-NBD104'),
                ('EXP-NBD114','Native Barcoding Expansion 13-24 EXP-NBD114'),
                ('EXP-NBD104+EXP-NBD114','Native Barcoding Expansions 1-12 and 13-24 EXP-NBD104+EXP-NBD114'),
                ('SQK-LSK109', 'Ligation Sequencing Kit, no barcodes SQK-LSK109'),
                # ('EXP-NBD103',''),
                # ('EXP-PBC001',''),
                # ('EXP-PBC096',''),
                # ('SQK-LWB001',''),
                # ('SQK-PBK004',''),
                # ('SQK-PCB109',''),
                # ('SQK-RAB201',''),
                # ('SQK-RAB204',''),
                # ('SQK-RBK001',''),
                # ('VSK-VMK001',''),
                # ('VSK-VMK002',''),
                # ('SQK-RLB001',''),
                ('SQK-RBK004','Rapid Barcoding Kit SQK-RBK004'),
                ('SQK-RPB004','Rapid PCR Barcoding Kit SQK-RPB004')]


    pk = AutoField()
    # path = PathField(index=True)
    # run = ForeignKeyField(Run, backref='_sample_sheet', unique=True, null=True)
    date = DateField(null=True, default=datetime.datetime.now())
    sequencing_kit = CharField(null=True)
    barcoding_kit = CharField(null=True, choices=BARCODES)
    library_id = CharField(null=True)

    @property
    def barcode_kit_barcodes(self):
        return {} #TODO

    @classmethod
    def get_unused_sheets(cls):
        return (cls.select()
                   .join(Run, JOIN.LEFT_OUTER)
                   .switch()
                   .where(Run.pk.is_null()))

    @classmethod
    def new_sheet_from_message(cls, sheet, run=None, log=logging.getLogger('porerefiner.models')):
        ss = cls.create(date=sheet.date.ToDatetime(),
                        barcoding_kit=sheet.sequencing_kit,
                        library_id=sheet.library_id)
        for sample in sheet.samples:
            Sample.create(sample_id=sample.sample_id,
                          accession=sample.accession,
                          barcode_id=sample.barcode_id,
                          organism=sample.organism,
                          extraction_kit=sample.extraction_kit,
                          comment=sample.comment,
                          user=sample.user,
                          samplesheet=ss)
        if run:
            run.sample_sheet = ss
            run.save()
        return ss

class Sample(PorerefinerModel):
    "A sample is an entry originally from a sample sheet"



    pk = AutoField()
    sample_id = CharField(null=False)
    accession = CharField(default="")
    barcode_id = CharField(null=False)
    #barcode_seq = CharField() #maybe set this when we load a sheet
    organism = CharField(default="")
    extraction_kit = CharField(default="")
    comment = CharField(default="")
    user = CharField(null=True)




    samplesheet = ForeignKeyField(SampleSheet, backref='samples')

    @property
    def barcode_seq(self):
        return self.samplesheet.barcode_kit_barcodes.get(self.barcode_id, "")

    @property
    def tags(self):
        return (Tag.select()
                   .join(TagJunction)
                   .where(TagJunction.sample == self))

    def tag(self, tag):
        ta, _ = Tag.get_or_create(name=tag)
        tj, _ = TagJunction.get_or_create(tag=ta, sample=self)
        return ta



class File(PorerefinerModel):
    "A file is a path on the filesystem"
    pk = AutoField()
    run = ForeignKeyField(Run, backref='files', null=True)
    sample = ForeignKeyField(Sample, backref='files', null=True)
    path = PathField(index=True, unique=True)
    checksum = CharField(index=True, null=True)
    last_modified = DateTimeField(default=datetime.datetime.now)
    exported = IntegerField(default=0)
    _jobs = ManyToManyField(Job, backref='files')

    @property
    def name(self):
        return self.path.name

    # @property
    # def jobs(self):
    #     return (Job.select()
    #                .join(JobFileJunction)
    #                .join(File)
    #                .where(File.pk == self.pk))

    def spawn(self, job_config):
        job = Job.create(status='READY', job_class=job_config.__class__.__name__, datadir=pathlib.Path(tempfile.mkdtemp()), file=self)
        self._jobs.add(job)
        return job

    def tag(self, tag):
        ta, _ = Tag.get_or_create(name=tag)
        tj, _ = TagJunction.get_or_create(tag=ta, file=self)
        return ta

    @property
    def jobs(self):
        yield from self._jobs_with_this_file_as_primary
        yield from self._jobs


class TagJunction(BaseModel):
    tag = ForeignKeyField(Tag, backref='junctions')
    # flowcell = ForeignKeyField(Flowcell, null=True, backref='tag_junctions')
    run = ForeignKeyField(Run, null=True, backref='tag_junctions')
    qa = ForeignKeyField(Qa, null=True, backref='tag_junctions')
    job = ForeignKeyField(Job, null=True, backref='tag_junctions')
    samplesheet = ForeignKeyField(SampleSheet, null=True, backref='tag_junctions')
    sample = ForeignKeyField(Sample, null=True, backref='tag_junctions')
    file = ForeignKeyField(File, null=True, backref='tag_junctions')

    # @classmethod
    # def relate(cls, target, tag):
    #     targ_cls = type(target)
    #     for field, ref_cls, _ in cls._meta.model_graph(backrefs=False, depth_first=False):
    #         if ref_cls is targ_cls:
    #             return cls.create

JobFileJunction = File._jobs.get_through_model()


REGISTRY = [Tag, Run, Qa, Job, SampleSheet, Sample, File, TagJunction, JobFileJunction]



import yaml
from collections import defaultdict
from pathlib import Path
from os import environ, makedirs
import logging

#from porerefiner.notifiers import REGISTRY, NOTIFIERS
import porerefiner.notifiers as notifiers
import porerefiner.jobs as jobs
import porerefiner.jobs.submitters as submitters

# config_file = Path(environ.get('POREREFINER_CONFIG', '/Users/justin.payne/.porerefiner/config.yml'))

def c(d):
    "Recursively convert this defaultdict to a dict"
    if isinstance(d, defaultdict):
        return {k:c(v) for k,v in d.items()}
    elif isinstance(d, Path):
        return str(d)
    else:
        return d


class Config:

    the_config = None

    # config_file = Path(environ.get('POREREFINER_CONFIG', '/Users/justin.payne/.porerefiner/config.yml'))

    @classmethod
    def __call__(cls, *args, **kwargs):
        if not cls.the_config:
            cls.the_config = super().__call__(cls, *args, **kwargs)
        return cls.the_config

    def __init__(self, config_file, client_only=False):

        self.config_file = config_file = Path(config_file)

        #Logging
        log = logging.getLogger('porerefiner.config')

        try:
            with open(config_file, 'r') as conf:
                self.config = yaml.safe_load(conf)
            log.info(f"Config file loaded from {config_file}")

        except OSError:
            log.warning(f'No config file at {config_file}, creating...')


            self.config = Config.new_config_file(config_file, client_only)

        except yaml.YAMLError as e:
            log.error(f"Couldn't read config file at {config_file}, error was:")
            log.error(e)
            quit(1)


        #Socket
        self.config['server']['socket'] = Path(environ.get('POREREFINER_SOCK', self.config['server']['socket']))
        log.info(f"PoreRefiner socket at {self.config['server']['socket']}")

        #Notifiers
        log.info("Loading notifiers...")
        for notifier_config in self.config.get('notifiers', []):
            log.info(f"Found notifier {notifier_config['config']['name']}")
            notifiers.REGISTRY[notifier_config['class']](**notifier_config['config'])
            #NOTIFIERS.append(notifier)

        #Submitters
        submitters.SUBMITTER = None
        log.info("Loading job submitters...")
        for submitter_config in self.config.get('submitters', []):
            log.info(f"Found submitter {submitter_config['class']}")
            submitter = submitters.REGISTRY[submitter_config['class']](**submitter_config['config'])
            #jobs
            log.info("Loading jobs...")
            for job_config in submitter_config.get('jobs', []):
                clss = jobs.CLASS_REGISTRY[job_config['class']]
                log.info(f"Found job {clss.__name__}")
                job = clss(submitter=submitter, **job_config['config'])

    @staticmethod
    def new_config_file(config_file, client_only=False, nanopore_path='/data', database_path=False, socket_path=False):

        porerefiner_dir = Path(config_file).parent


        tree = lambda: defaultdict(tree) #this is a trick for defining a recursive defaultdict
        defaults = tree()

        defaults['server']['socket'] = socket_path or porerefiner_dir / 'socket' # '/Users/justin.payne/.porerefiner/socket'
        defaults['server']['use_ssl'] = False

        if not client_only:
            defaults['porerefiner']['log_level'] = logging.INFO
            defaults['porerefiner']['run_polling_interval'] = 600
            defaults['porerefiner']['job_polling_interval'] = 1800
            defaults['database']['path'] = database_path or porerefiner_dir / 'database.db' # '/Users/justin.payne/.porerefiner/database.db'
            defaults['database']['pragmas']['foreign_keys'] = 1
            defaults['database']['pragmas']['journal_mode'] = 'wal'
            defaults['database']['pragmas']['cache_size'] = 1000
            defaults['database']['pragmas']['ignore_check_constraints'] = 0
            defaults['database']['pragmas']['synchronous'] = 0
            defaults['nanopore']['path'] = nanopore_path
            defaults['nanopore']['api'] = "localhost:9501"

            try:
                from pynotifier import Notification
                defaults['notifiers'] = [{'class':'ToastNotifier', 'config':dict(name='Default notifier', max=3)}]
            except ImportError: # pynotifier not installed, so no need to create a Toast notifier
                pass

# {'class':'HpcSubmitter', 'config':dict(login_host="login1-raven2.fda.gov",
#                                                                             username="nanopore",
#                                                                             private_key_path=".ssh/id_rsa",
#                                                                             known_hosts_path=".ssh/known_hosts",
#                                                                             scheduler="uge",
#                                                                             queue="service.q"),
#                                         'jobs':[{'class':'GuppyJob', 'config':dict(num_cores=16)}]
#                                         },
#                                     {'class':'Epi2meSubmitter', 'config':dict(api_key=''),
#                                     'jobs':[{'class':'EpiJob', 'config':dict()}]
#                                     }

            defaults['submitters'] = []




        defaults = c(defaults)

        makedirs(config_file.parent, exist_ok=True)

        # defaults['']['']
        with open(config_file, 'w') as conf:
            yaml.dump(defaults, conf)

        return defaults

#Jobs
# log.info("Loading jobs...")
# for job_config in config.get('jobs', []):
#     log.info(f"Found job {job_config['class']}")
#     jobs.REGISTRY[job_config['class']](**job_config['config'])

# def add_notifier_stub(notifier_cls):
#     cp = config.copy() #we don't want the stub to actually take effect until reload
#     cp['notifiers'].append(notifier_cls.toStub())
#     with open(config_file, 'w') as conf:
#         yaml.dump(cp, conf)





from unittest import TestCase, skip

from tests import paths, with_database, TestBase

from porerefiner import notifiers

class TestNotifierBaseClass(TestCase):

    def test_number_of_classes(self):
        self.assertGreaterEqual(len(notifiers.REGISTRY), 3)



import asyncio
import aiohttp
import logging
from dataclasses import dataclass

log = logging.getLogger('porerefiner.http_notifier')

from porerefiner.notifiers import Notifier

@dataclass
class HttpCallbackNotifier(Notifier):

    sample_name: str = "http://sample.url/api/"


    async def notify(self, run, state, message): #TODO
        "Send a message to the configured URL via http-form POST"
        log.info(f"Notifying {self.name}...")
        async with aiohttp.ClientSession() as session:
            async with self.session.post(self.name, data=dict(run=run, state=state, message=message)) as response:
                log.info(response.status)
                log.info(response.text)



from unittest import TestCase, skip
from unittest.mock import patch, Mock, MagicMock
from mock import AsyncMock

from tests import paths, with_database, TestBase

from porerefiner.app.main import app as _app
from porerefiner.cli_utils import server




class TestFlask(TestCase):

    def setUp(self):
        global app
        app = _app.test_client()


    @skip("can't mock")
    @patch('porerefiner.app.main.server')
    def test_attach_run(self, mock):
        mock.__enter__ = mock
        mock.__exit__ = mock
        app.post('/api/runs/1/attach')
        mock.AttachSheetToRun.assert_called_once()

    @skip("can't mock")
    @patch('porerefiner.app.main.server')
    def test_list_runs(self, mock):
        mock.return_value.__enter__.return_value = m = AsyncMock(__iter__=Mock())
        #mock.return_value.__exit__ = mock
        app.get('/api/runs/')



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: minknow/rpc/manager.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import instance_pb2 as minknow_dot_rpc_dot_instance__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='minknow/rpc/manager.proto',
  package='ont.rpc.manager',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x19minknow/rpc/manager.proto\x12\x0font.rpc.manager\x1a\x1aminknow/rpc/instance.proto\"\x14\n\x12ListDevicesRequest\"\xb3\x03\n\x13ListDevicesResponse\x12\x10\n\x08inactive\x18\x01 \x03(\t\x12\x0f\n\x07pending\x18\x02 \x03(\t\x12\x41\n\x06\x61\x63tive\x18\x03 \x03(\x0b\x32\x31.ont.rpc.manager.ListDevicesResponse.ActiveDevice\x1ap\n\x08RpcPorts\x12\x0f\n\x07jsonrpc\x18\x01 \x01(\r\x12\x16\n\x0ejson_websocket\x18\x02 \x01(\r\x12\x0e\n\x06secure\x18\x03 \x01(\r\x12\x15\n\rinsecure_grpc\x18\x04 \x01(\r\x12\x14\n\x0cinsecure_web\x18\x05 \x01(\r\x1a$\n\x0c\x44\x65viceLayout\x12\t\n\x01x\x18\x01 \x01(\x05\x12\t\n\x01y\x18\x02 \x01(\x05\x1a\x9d\x01\n\x0c\x41\x63tiveDevice\x12\x0c\n\x04name\x18\x01 \x01(\t\x12<\n\x05ports\x18\x02 \x01(\x0b\x32-.ont.rpc.manager.ListDevicesResponse.RpcPorts\x12\x41\n\x06layout\x18\x03 \x01(\x0b\x32\x31.ont.rpc.manager.ListDevicesResponse.DeviceLayout\"\x17\n\x15GetVersionInfoRequest\"\xab\x02\n\x16GetVersionInfoResponse\x12H\n\x07minknow\x18\x01 \x01(\x0b\x32\x37.ont.rpc.instance.GetVersionInfoResponse.MinknowVersion\x12\x11\n\tprotocols\x18\x02 \x01(\t\x12\x1c\n\x14\x64istribution_version\x18\x03 \x01(\t\x12X\n\x13\x64istribution_status\x18\x04 \x01(\x0e\x32;.ont.rpc.instance.GetVersionInfoResponse.DistributionStatus\x12\x1b\n\x13guppy_build_version\x18\x05 \x01(\t\x12\x1f\n\x17guppy_connected_version\x18\x06 \x01(\t2\xd4\x01\n\x0eManagerService\x12[\n\x0clist_devices\x12#.ont.rpc.manager.ListDevicesRequest\x1a$.ont.rpc.manager.ListDevicesResponse\"\x00\x12\x65\n\x10get_version_info\x12&.ont.rpc.manager.GetVersionInfoRequest\x1a\'.ont.rpc.manager.GetVersionInfoResponse\"\x00\x62\x06proto3')
  ,
  dependencies=[minknow_dot_rpc_dot_instance__pb2.DESCRIPTOR,])




_LISTDEVICESREQUEST = _descriptor.Descriptor(
  name='ListDevicesRequest',
  full_name='ont.rpc.manager.ListDevicesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=74,
  serialized_end=94,
)


_LISTDEVICESRESPONSE_RPCPORTS = _descriptor.Descriptor(
  name='RpcPorts',
  full_name='ont.rpc.manager.ListDevicesResponse.RpcPorts',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='jsonrpc', full_name='ont.rpc.manager.ListDevicesResponse.RpcPorts.jsonrpc', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='json_websocket', full_name='ont.rpc.manager.ListDevicesResponse.RpcPorts.json_websocket', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='secure', full_name='ont.rpc.manager.ListDevicesResponse.RpcPorts.secure', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='insecure_grpc', full_name='ont.rpc.manager.ListDevicesResponse.RpcPorts.insecure_grpc', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='insecure_web', full_name='ont.rpc.manager.ListDevicesResponse.RpcPorts.insecure_web', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=222,
  serialized_end=334,
)

_LISTDEVICESRESPONSE_DEVICELAYOUT = _descriptor.Descriptor(
  name='DeviceLayout',
  full_name='ont.rpc.manager.ListDevicesResponse.DeviceLayout',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='ont.rpc.manager.ListDevicesResponse.DeviceLayout.x', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='y', full_name='ont.rpc.manager.ListDevicesResponse.DeviceLayout.y', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=336,
  serialized_end=372,
)

_LISTDEVICESRESPONSE_ACTIVEDEVICE = _descriptor.Descriptor(
  name='ActiveDevice',
  full_name='ont.rpc.manager.ListDevicesResponse.ActiveDevice',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='ont.rpc.manager.ListDevicesResponse.ActiveDevice.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ports', full_name='ont.rpc.manager.ListDevicesResponse.ActiveDevice.ports', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='layout', full_name='ont.rpc.manager.ListDevicesResponse.ActiveDevice.layout', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=375,
  serialized_end=532,
)

_LISTDEVICESRESPONSE = _descriptor.Descriptor(
  name='ListDevicesResponse',
  full_name='ont.rpc.manager.ListDevicesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='inactive', full_name='ont.rpc.manager.ListDevicesResponse.inactive', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pending', full_name='ont.rpc.manager.ListDevicesResponse.pending', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='active', full_name='ont.rpc.manager.ListDevicesResponse.active', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_LISTDEVICESRESPONSE_RPCPORTS, _LISTDEVICESRESPONSE_DEVICELAYOUT, _LISTDEVICESRESPONSE_ACTIVEDEVICE, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=97,
  serialized_end=532,
)


_GETVERSIONINFOREQUEST = _descriptor.Descriptor(
  name='GetVersionInfoRequest',
  full_name='ont.rpc.manager.GetVersionInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=534,
  serialized_end=557,
)


_GETVERSIONINFORESPONSE = _descriptor.Descriptor(
  name='GetVersionInfoResponse',
  full_name='ont.rpc.manager.GetVersionInfoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='minknow', full_name='ont.rpc.manager.GetVersionInfoResponse.minknow', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='protocols', full_name='ont.rpc.manager.GetVersionInfoResponse.protocols', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='distribution_version', full_name='ont.rpc.manager.GetVersionInfoResponse.distribution_version', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='distribution_status', full_name='ont.rpc.manager.GetVersionInfoResponse.distribution_status', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='guppy_build_version', full_name='ont.rpc.manager.GetVersionInfoResponse.guppy_build_version', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='guppy_connected_version', full_name='ont.rpc.manager.GetVersionInfoResponse.guppy_connected_version', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=560,
  serialized_end=859,
)

_LISTDEVICESRESPONSE_RPCPORTS.containing_type = _LISTDEVICESRESPONSE
_LISTDEVICESRESPONSE_DEVICELAYOUT.containing_type = _LISTDEVICESRESPONSE
_LISTDEVICESRESPONSE_ACTIVEDEVICE.fields_by_name['ports'].message_type = _LISTDEVICESRESPONSE_RPCPORTS
_LISTDEVICESRESPONSE_ACTIVEDEVICE.fields_by_name['layout'].message_type = _LISTDEVICESRESPONSE_DEVICELAYOUT
_LISTDEVICESRESPONSE_ACTIVEDEVICE.containing_type = _LISTDEVICESRESPONSE
_LISTDEVICESRESPONSE.fields_by_name['active'].message_type = _LISTDEVICESRESPONSE_ACTIVEDEVICE
_GETVERSIONINFORESPONSE.fields_by_name['minknow'].message_type = minknow_dot_rpc_dot_instance__pb2._GETVERSIONINFORESPONSE_MINKNOWVERSION
_GETVERSIONINFORESPONSE.fields_by_name['distribution_status'].enum_type = minknow_dot_rpc_dot_instance__pb2._GETVERSIONINFORESPONSE_DISTRIBUTIONSTATUS
DESCRIPTOR.message_types_by_name['ListDevicesRequest'] = _LISTDEVICESREQUEST
DESCRIPTOR.message_types_by_name['ListDevicesResponse'] = _LISTDEVICESRESPONSE
DESCRIPTOR.message_types_by_name['GetVersionInfoRequest'] = _GETVERSIONINFOREQUEST
DESCRIPTOR.message_types_by_name['GetVersionInfoResponse'] = _GETVERSIONINFORESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ListDevicesRequest = _reflection.GeneratedProtocolMessageType('ListDevicesRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTDEVICESREQUEST,
  '__module__' : 'minknow.rpc.manager_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.manager.ListDevicesRequest)
  })
_sym_db.RegisterMessage(ListDevicesRequest)

ListDevicesResponse = _reflection.GeneratedProtocolMessageType('ListDevicesResponse', (_message.Message,), {

  'RpcPorts' : _reflection.GeneratedProtocolMessageType('RpcPorts', (_message.Message,), {
    'DESCRIPTOR' : _LISTDEVICESRESPONSE_RPCPORTS,
    '__module__' : 'minknow.rpc.manager_pb2'
    # @@protoc_insertion_point(class_scope:ont.rpc.manager.ListDevicesResponse.RpcPorts)
    })
  ,

  'DeviceLayout' : _reflection.GeneratedProtocolMessageType('DeviceLayout', (_message.Message,), {
    'DESCRIPTOR' : _LISTDEVICESRESPONSE_DEVICELAYOUT,
    '__module__' : 'minknow.rpc.manager_pb2'
    # @@protoc_insertion_point(class_scope:ont.rpc.manager.ListDevicesResponse.DeviceLayout)
    })
  ,

  'ActiveDevice' : _reflection.GeneratedProtocolMessageType('ActiveDevice', (_message.Message,), {
    'DESCRIPTOR' : _LISTDEVICESRESPONSE_ACTIVEDEVICE,
    '__module__' : 'minknow.rpc.manager_pb2'
    # @@protoc_insertion_point(class_scope:ont.rpc.manager.ListDevicesResponse.ActiveDevice)
    })
  ,
  'DESCRIPTOR' : _LISTDEVICESRESPONSE,
  '__module__' : 'minknow.rpc.manager_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.manager.ListDevicesResponse)
  })
_sym_db.RegisterMessage(ListDevicesResponse)
_sym_db.RegisterMessage(ListDevicesResponse.RpcPorts)
_sym_db.RegisterMessage(ListDevicesResponse.DeviceLayout)
_sym_db.RegisterMessage(ListDevicesResponse.ActiveDevice)

GetVersionInfoRequest = _reflection.GeneratedProtocolMessageType('GetVersionInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETVERSIONINFOREQUEST,
  '__module__' : 'minknow.rpc.manager_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.manager.GetVersionInfoRequest)
  })
_sym_db.RegisterMessage(GetVersionInfoRequest)

GetVersionInfoResponse = _reflection.GeneratedProtocolMessageType('GetVersionInfoResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETVERSIONINFORESPONSE,
  '__module__' : 'minknow.rpc.manager_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.manager.GetVersionInfoResponse)
  })
_sym_db.RegisterMessage(GetVersionInfoResponse)



_MANAGERSERVICE = _descriptor.ServiceDescriptor(
  name='ManagerService',
  full_name='ont.rpc.manager.ManagerService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=862,
  serialized_end=1074,
  methods=[
  _descriptor.MethodDescriptor(
    name='list_devices',
    full_name='ont.rpc.manager.ManagerService.list_devices',
    index=0,
    containing_service=None,
    input_type=_LISTDEVICESREQUEST,
    output_type=_LISTDEVICESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_version_info',
    full_name='ont.rpc.manager.ManagerService.get_version_info',
    index=1,
    containing_service=None,
    input_type=_GETVERSIONINFOREQUEST,
    output_type=_GETVERSIONINFORESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_MANAGERSERVICE)

DESCRIPTOR.services_by_name['ManagerService'] = _MANAGERSERVICE

# @@protoc_insertion_point(module_scope)




import asyncio
import logging
import timeit

from pathlib import Path
from uuid import uuid4
from random import choice, randint
from tempfile import TemporaryDirectory
from threading import Thread
from os import makedirs

import click

from peewee import SqliteDatabase
from porerefiner import models
from porerefiner.fsevents import start_fs_watchdog

def start_pr(path):
    

    loop = asyncio.new_event_loop()



    def l(loop):
        logging.basicConfig(stream=open('/dev/null', 'w'))
        db = SqliteDatabase(":memory:", pragmas={'foreign_keys':1}, autoconnect=False)
        db.bind(models.REGISTRY, bind_refs=False, bind_backrefs=False)
        db.connect()
        db.create_tables(models.REGISTRY)
        asyncio.set_event_loop(loop)
        loop.run_forever()
        click.echo(f"{models.Run.select().count()} runs, {models.File.select().count()} files")
        db.drop_tables(models.REGISTRY)
        db.close()


    # future = asyncio.ensure_future(start_fs_watchdog(path))
    # loop.call_soon_threadsafe(future)

    future = asyncio.run_coroutine_threadsafe(start_fs_watchdog(path, loop=loop), loop)

    t = Thread(target=l, args=[loop])
    t.start()

    return loop, future


def stop_pr(loop, future):

    loop.stop()
    future.cancel()
    # while loop.is_running():
    #     pass
    # loop.close()

    

@click.command()
@click.argument('n', default=3000)
def main(n, trials=5):

    with TemporaryDirectory() as t:

        path = Path(t) / "EXPERIMENTEXPERIMENT" / "SAMPLESAMPLE"

        for run in ['RUN_1_RUN', 'RUN_2_RUN']:
            for sub in ['fastq_pass', 'fastq_fail']:
                makedirs(path / run / sub)
    
        def fs_spray(n):
            print(f"opening {n} files")
            fps = [open(path / choice(['RUN_1_RUN', 'RUN_2_RUN']) / choice(['fastq_pass', 'fastq_fail', '']) / (str(uuid4()) + '.bin'), 'wb', buffering=0) for _ in range(n)]
            print(f"100,000 {len(bytes(str(uuid4()), 'utf-8'))}-byte writes to {n} files")
            for _ in range(100000):
                choice(fps).write(bytes(str(uuid4()), 'utf-8'))
            [fp.close() for fp in fps]
            # print(f"100,000 {len(bytes(str(uuid4()), 'utf-8'))}-byte writes to {n} files")

        click.echo("Spraying filesystem for baseline...")
        basetime = timeit.timeit("fs_spray(n)", number=trials, globals=locals())
        click.echo(f"{basetime:03} seconds.")
        click.echo("Setting up filesystem event watcher...")
        loop, future= start_pr(t)
        click.echo("Spraying filesystem under watchdog...")
        realtime = timeit.timeit("fs_spray(n)", number=trials, globals=locals())
        click.echo(f"{realtime:03} seconds.")
        click.echo("Bringing down watchdog thread...")
        stop_pr(loop, future)
        #click.echo(basetime)
        #click.echo(realtime)
        click.echo(f"{realtime - basetime:02} seconds difference.")

if __name__ == '__main__':
    quit(main())



from asyncio import run
from flask import Flask, request, current_app
from google.protobuf.json_format import MessageToJson

import json


from porerefiner.cli_utils import server
from porerefiner.samplesheets import load_from_csv
from porerefiner.protocols.porerefiner.rpc.porerefiner_pb2 import RunRequest, RunListRequest, RunAttachRequest, RunRsyncRequest

from porerefiner.app import app



@app.route('/api/form/attach/submit', methods=['POST', ])
@app.route('/api/runs/<int:run_id>/attach', methods=['POST,'])
def attach_to_run(run_id=None):
    if not run_id:
        run_id = request.form.get('run_id', None)
    file = request.files.get('sample_sheet')
    message = load_from_csv(file)
    async def attach_runner(run_id, message):
        with server(current_app.config['config_file']) as serv:
            return await serv.AttachSheetToRun(RunAttachRequest(name=run_id, sheet=message))
    resp = run(attach_runner(run_id, message))
    return json.dumps({'success':True}), 200, {'ContentType':'application/json'}



@app.route('/api/runs/<int:run_id>')
def get_run(run_id):
    "Get a single run"
    async def run_getter(run_id):
        with server(current_app.config['config_file']) as serv:
            return await serv.GetRunInfo(RunRequest(id = run_id))
    resp = run(run_getter(run_id))
    return MessageToJson(resp), 200, {'ContentType':'application/json'}

@app.route('/api/runs')
def list_runs():
    #all = request.args.get('all', False)
    tags = [str(t) for t in request.args.getlist('tags')]
    async def list_run_runner(all, tags):
        with server(current_app.config['config_file']) as serv:
            return await serv.GetRuns(RunListRequest(all=True, tags=tags))

    resp = run(list_run_runner(all, tags))
    return MessageToJson(resp), 200, {'ContentType':'application/json'}





import asyncio

from unittest import TestCase, skip
from unittest.mock import Mock, patch

import porerefiner.jobs
import porerefiner.jobs.submitters

from porerefiner.jobs import submit_job, poll_active_job, poll_jobs

from tests import *
from hypothesis import given
from hypothesis.strategies import *

def run(task):
    loop = asyncio.get_event_loop()
    return loop.run_until_complete(task)


class TestJobSubmission(TestCase):

    @patch('porerefiner.jobs.submitters.logging')
    @given(job=Model.Jobs())
    @with_database
    def test_submit_job(self, log, job):
        job.save()
        assert run(submit_job(job))

    @patch('porerefiner.jobs.logging')
    @given(job=Model.Jobs())
    @with_database
    def test_poll_active_job(self, log, job):
        job.save()
        assert run(poll_active_job(job))

    @patch('porerefiner.jobs.logging')
    @given(jobs=lists(Model.Jobs()))
    @with_database
    def test_poll_jobs(self, log, jobs):
        result, _, _ = run(poll_jobs(jobs, jobs))
        self.assertEqual(result, len(jobs) * 2)


"Custom argument handler types for Click"

import click
import csv
import datetime
import json
import sys
from pathlib import Path
from contextlib import contextmanager
from collections import defaultdict
from dataclasses import MISSING
from functools import wraps, partial
from io import TextIOWrapper
from typing import List


from grpclib.client import Channel, GRPCError

from tabulate import tabulate
from xml.etree import ElementTree as xml
from xml.dom import minidom
from sys import stderr

from porerefiner.protocols.porerefiner.rpc.porerefiner_grpc import PoreRefinerStub
from porerefiner.protocols.porerefiner.rpc.porerefiner_pb2 import SampleSheet

#these two functions are hooks in case we decide to store only relative paths.
#paths going into the database will be relativized; paths going out will be
#absolutized.
#Whatever they do, they need to invert each other
def relativize_path(path):
    if isinstance(path, str):
        return Path(path)
    return path

def absolutize_path(path):
    if hasattr(path, '__fspath__'):
        return str(path)
    return path

class RunID:
    "can be either a string name or a numeric id"

    RUN_NAME = 'RUN_NAME'
    RUN_ID = 'RUN_ID'

    def __init__(self, the_value):
        if isinstance(the_value, str):
            self.val_type = self.RUN_NAME
        elif isinstance(the_value, int):
            self.val_type = self.RUN_ID
        else:
            raise ValueError("value must be str or int")



class ValidRunID(click.ParamType):

    def convert(self, value, param, ctx):
        try:
            return int(value)
        except ValueError:
            return str(value)

VALID_RUN_ID = ValidRunID()

class PathPath(click.Path):

    def convert(self, value, param, ctx):
        val = super().convert(value, param, ctx)
        return Path(val)


def handle_connection_errors(func):
    @wraps(func)
    def wrapper(*a, **k):
        try:
            return func(*a, **k)
        except ConnectionRefusedError:
            print("ERROR: Connection to porerefiner service refused. Is the service running?", file=stderr)
            quit(61)
        except FileNotFoundError as e:
            print(f"ERROR: Socket file not found at configured path. Has the service ever been started?", file=stderr)
            quit(e.errno)
    return wrapper


# Channel context manager for CLI utils

@contextmanager
def server(config_file):
    from porerefiner.config import Config
    config = Config(config_file, client_only=True).config
    channel = Channel(path=config['server']['socket'], ssl=config['server']['use_ssl'])
    client = PoreRefinerStub(channel)
    try:
        yield client
    except GRPCError as e:
        raise
    finally:
        channel.close()

#formatters for run output

@contextmanager
def hr_formatter(extend=False):
    rec = []
    def print_run(run):
        rec.append(dict(id=run.id,
                        name=run.name,
                        nickname=run.mnemonic_name,
                        status=run.status,
                        samples=len(run.samples),
                        files=sum([len(sam.files) for sam in run.samples]) + len(run.files),
                        tags=",".join(run.tags)))
        if extend:
            rec[-1]['path'] = run.path
            rec[-1]['flowcell'] = run.flowcell_id
            for sample in run.samples:
                rec.append(dict(id=sample.id,
                                name=sample.name,
                                nickname=sample.accession,
                                barcode_id=sample.barcode_id,
                                organism=sample.organism,
                                comment=sample.comment,
                                user=sample.user,
                                files=len(sample.files),
                                tags=",".join(sample.tags)))
            for file in run.files:
                rec.append(dict(id='-',
                                name=file.name,
                                path=file.path,
                                status=('UNREADY', 'READY')[file.ready],
                                tags=",".join(file.tags)))
    yield print_run
    print(tabulate(rec, headers="keys"))

class MessageAwareEncoder(json.JSONEncoder):

    def default(self, o):
        from google.protobuf.json_format import MessageToDict
        from google.protobuf.message import Message
        if isinstance(o, Message):
            return MessageToDict(o)
        return json.JSONEncoder.default(self, o)


@contextmanager
def json_formatter(extend=False):
    rec = []
    def print_run(run):
        rec.append(run)
    yield print_run
    click.echo(json.dumps(rec, cls=MessageAwareEncoder, indent=2))

@contextmanager
def xml_formatter(extend=False): #TODO
    rec = xml.Element('root')
    def print_run(run):
        r = xml.SubElement(rec, 'run')
        xml.SubElement(r, 'id').text = str(run.id)
        xml.SubElement(r, 'name').text = run.name
        xml.SubElement(r, 'nickname').text = run.mnemonic_name
        xml.SubElement(r, 'status').text = run.status
        xml.SubElement(r, 'path').text = run.path
        xml.SubElement(r, 'basecallingModel').text = run.basecalling_model

        s = xml.SubElement(r, 'samples')
        for sample in run.samples:
            sa = xml.SubElement(s, 'sample')
            xml.SubElement(sa, 'id').text = str(sample.id)
            xml.SubElement(sa, 'name').text = sample.name
            xml.SubElement(sa, 'accession').text = sample.accession
            xml.SubElement(sa, 'barcodeId').text = sample.barcode_id
            xml.SubElement(sa, 'organism').text = sample.organism
            xml.SubElement(sa, 'comment').text = sample.comment
            xml.SubElement(sa, 'user').text = sample.user
            f = xml.SubElement(sa, 'files')
            for file in sample.files:
                fi = xml.SubElement(f, 'file')
                xml.SubElement(fi, 'name').text = file.name
                xml.SubElement(fi, 'path').text = file.path
                xml.SubElement(fi, 'status').text = ('UNREADY', 'READY')[file.ready]
                xml.SubElement(fi, 'hash').text = file.hash
                t = xml.SubElement(fi, 'tags')
                for tag in file.tags:
                    xml.SubElement(t, 'tag').text = tag
            t = xml.SubElement(sa, 'tags')
            for tag in sample.tags:
                xml.SubElement(t, 'tag').text = tag


        f = xml.SubElement(r, 'files')
        for file in run.files:
            fi = xml.SubElement(f, 'file')
            xml.SubElement(fi, 'name').text = file.name
            xml.SubElement(fi, 'path').text = file.path
            xml.SubElement(fi, 'status').text = ('UNREADY', 'READY')[file.ready]
            xml.SubElement(fi, 'hash').text = file.hash
            t = xml.SubElement(fi, 'tags')
            for tag in file.tags:
                xml.SubElement(t, 'tag').text = tag

        t = xml.SubElement(r, 'tags')
        for tag in run.tags:
            xml.SubElement(t, 'tag').text = tag

    yield print_run
    pretty = minidom.parseString(xml.tostring(rec, 'utf-8'))
    click.echo(pretty.toprettyxml(indent='  '))
    #xml.ElementTree(rec).write(sys.stdout, encoding='unicode', xml_declaration=True)




class Email:
    pass

class Url:
    pass

class PathStr:
    pass


def render_dataclass(the_class):
    "Inspect dataclass fields and print some helpful text"
    sample_values = defaultdict(lambda: "<some_value>")
    sample_values.update({str:'"some string"',
                          int:"123",
                          float:"123.45",
                          dict:"{key1:val1, key2:val2}",
                          Email:"eee@mail.net",
                          Url:"http://server.serv/path",
                          PathStr:"path/to/some/file",
                          List[str]:"[<string>,...]",
                          List[Email]:"[aaa@mail.net, bbb@mail.net, ...]",
                          List[int]:"[<integer>,...]",
                          List[Url]:"[http://server.co, http://serv.cc, ...]",
                          List[PathStr]:"[path/aaa, path/bbb, ...]"})
    nw = max([len(field.name) for field in the_class.__dataclass_fields__.values()])
    tw = max([len(str(field.type)) for field in the_class.__dataclass_fields__.values() if field.name != 'submitter'])
    options = f"{'param_name':<{nw}}\t{'type':<{tw}}\texample"
    for field in the_class.__dataclass_fields__.values():
        if field.name != 'submitter':
            name = f"{field.name:<{nw}}"
            _type = f"{str(field.type):<{tw}}"
            example = (str(field.default), sample_values[field.type])[type(field.default) is not MISSING]
            options += f"\n\t{name}\t{_type}\t{example}"
    # options += "\n\t".join([f"{field.name:<{nw}}\t{str(field.type):<{tw}}\t{sample_values[field.type]}" for field in the_class.__dataclass_fields__.values() if field.name is not 'submitter'])
    return f"""
classname: {the_class.__name__}

{the_class.__doc__}

configurable options:

\t{options}
"""
